import csv, math


LATITUDE = 11
LONGITUDE = 12
CODE_INSEE = 10
REGION =2


def lire_fichier_csv(nom_fichier):
    table = []
    with open(nom_fichier, mode='r', encoding='utf-8') as fichier:
        iterateur_csv = csv.reader(fichier, delimiter=';')
        for ligne in iterateur_csv:
            table.append(ligne)
    return table
            

def affiche_index_champs(noms_champs):
    for i, champ in enumerate(noms_champs):
        print(f"{i:>3}: " + champ)


def affiche_liste(liste):
    for element in liste:
        print(element)


def sans_doublons(table):
    codes_insee = set()
    liste_villes = []
    for ville in table:
        code = ville[CODE_INSEE]
        if not code in codes_insee:
            liste_villes.append(ville)
            codes_insee.add(code)
    return liste_villes


def convertir_ligne(ville):
    try:
        ville[LATITUDE] = float(ville[LATITUDE].replace(',', '.'))
    except ValueError:
        return False
    try:
        ville[LONGITUDE] = float(ville[LONGITUDE].replace(',', '.'))
    except ValueError:
        return False
    return True


def convertir_table(table):
    donnees_correctes = []
    for ville in table:
        if convertir_ligne(ville):
            donnees_correctes.append(ville)
    return donnees_correctes


def intervalle_latitude(table):
    return (min(ville[LATITUDE] for ville in table), max(ville[LATITUDE] for ville in table))


def intervalle_longitude(table):
    return (min(ville[LONGITUDE] for ville in table), max(ville[LONGITUDE] for ville in table))


def ensemble_regions(table):
    regions = set()
    for ville in table:
        if not ville[2] in regions:
            regions.add(ville[REGION])
    return regions


def villes_metropole(table):
    metropole = []
    for ville in table:
        if ville[REGION] != "Corse":
            metropole.append(ville)
    return metropole


def villes_est(table, longitude):
    return [ville for ville in table if ville[LONGITUDE] >= longitude]


def villes_ouest(table, longitude):
    return [ville for ville in table if ville[LONGITUDE] <= longitude]


def villes_sud(table, latitude):
    return [ville for ville in table if ville[LATITUDE] <= latitude]


def villes_nord(table, latitude):
    return [ville for ville in table if ville[LATITUDE] >= latitude]


def distance_villes(villeA, villeB):
    dLon = math.pi * (villeB[LONGITUDE] - villeA[LONGITUDE]) / 180
    latA = math.pi * villeA[LATITUDE] / 180
    latB = math.pi * villeB[LATITUDE] / 180
    valeur = math.sin(latA) * math.sin(latB) + math.cos(latA) * math.cos(latB) * math.cos(dLon)
    if valeur > 1:# les erreurs d'arrondi peuvent provoqué un dépassement
        valeur = 1.0
    d = math.acos(valeur)
    return d * 6378.137


def somme_distance_autres_villes(ville, table):
    somme = 0
    for i, v in enumerate(table):
        d = distance_villes(ville, v)
        somme += d
    return somme
 

def calcule_sommes_distances(table):
    for i, ville in enumerate(table):
        ville.append(somme_distance_autres_villes(ville, table))
        if (i + 1) % 100 == 0:
            print(f"{i + 1} villes traitées.")


def trier_par_colonne(table, colonne):
    table.sort(key=lambda ville: ville[colonne])

    
table = lire_fichier_csv("EUCircos_Regions_departements_circonscriptions_communes_gps.csv")
noms_champs = table[0]
del table[0]


affiche_index_champs(noms_champs)
print(len(table), "villes dans le fichier.")
print()


table2 = convertir_table(table)
table3 = sans_doublons(table2)

print(len(table3), "villes différentes avec des informations utilisables.")
print()

print("Plage de latitudes :", intervalle_latitude(table3))
print("Plage de longitudes :", intervalle_longitude(table3))
print()

print("Régions présentes")
print(ensemble_regions(table3))
table4 = villes_metropole(table3)
print()

print(len(table4), "villes différentes de métropole avec des informations utilisables.")
print()

print("Plage de latitudes :", intervalle_latitude(table4))
print("Plage de longitudes :", intervalle_longitude(table4))
print("Villes les plus à l'est :")
affiche_liste(villes_est(table4, 8.17))
print("Villes les plus à l'ouest :")
affiche_liste(villes_ouest(table4, -5.1))
print("Villes les plus au sud :")
affiche_liste(villes_sud(table4, 42.37))
print("Villes les plus au nord :")
affiche_liste(villes_nord(table4, 51.08))
print()

#print(table[0])
#print(somme_distance_autres_villes(table[0], table4))
calcule_sommes_distances(table4)
trier_par_colonne(table4, 14)

for i in range(10):
    print(table4[i])
